import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.transform.stream.StreamSource;

import javax.xml.bind.ValidationEvent;
import javax.xml.bind.ValidationException;
import javax.xml.bind.Validator;
import javax.xml.bind.util.ValidationEventCollector;

import java.io.*;
import java.util.*;
import java.sql.*;
import java.net.URL;
import java.lang.*;
import java.math.*;

import xml.config.*;
import xml.input.*;
import xml.msmini.*;
import xml.masslynx.*;


//------------------------------------------------------------------
// CLASS:        mzdataexport
// LAST CHANGE:  02-11-2005
// COMMENT:      exports the selected subset of data from the MeMo 
//               DB into the mzData format
//------------------------------------------------------------------
public class mzdataexport
{

private static final boolean DEBUG = true;
private static final boolean QUERY = true;
private static final int MAXARRAY  = 4000;
private static final int MAXSPEC   = 10;


// *******************************************************************
//                        * * * MAIN * * *
// *******************************************************************
public static void main (String args[]) 
{

try {

long startTime = System.currentTimeMillis();
boolean valid = false;



// ----------------- prepare for XML unmarshalling ---------------------
if (DEBUG) System.out.print("Preparing for CONFIG.XML unmarshalling... ");

// --- create JAXBContext object to provide entry point to JAXB API
JAXBContext jc = JAXBContext.newInstance("xml.config");

// --- create Unmarshaller object to control the process of unmarshalling
Unmarshaller unmarshaller = jc.createUnmarshaller();
unmarshaller.setValidating(true);

if (DEBUG) System.out.println("Done!\n");
// ---------------------------------------------------------------------


// ----------------------- unmarshal config.xml ------------------------
if (DEBUG) System.out.print("Unmarshalling CONFIG.XML... ");
ConfigType configContent = (ConfigType) unmarshaller.unmarshal(new File("config.xml"));
if (DEBUG) System.out.println("Done!\n");

if (DEBUG) System.out.print("Reading the parameters from CONFIG.XML... ");

// --- parameters for DB connection
ConfigType.DatabaseType db = configContent.getDatabase();

String url      = db.getUrl().trim();
String driver   = db.getDriver().trim();
String user     = db.getUser().trim();
String password = db.getPassword().trim();


if (DEBUG) 
{
  System.out.println("Done!\n");

  System.out.println("CONFIG.XML content:");
  System.out.println("-------------------");

  System.out.println("Database: " + url);
  System.out.println("User:     " + user);
}
// ---------------------------------------------------------------------



// ----------------- prepare for XML unmarshalling ---------------------
if (DEBUG) System.out.print("Preparing for INPUT.XML unmarshalling... ");
jc = JAXBContext.newInstance("xml.input");
unmarshaller = jc.createUnmarshaller();
unmarshaller.setValidating(true);
if (DEBUG) System.out.println("Done!\n");
// ---------------------------------------------------------------------



// ----------------------- unmarshal input.xml -------------------------
if (DEBUG) System.out.print("Unmarshalling INPUT.XML... ");
InputType inputContent = (InputType) unmarshaller.unmarshal(new File("input.xml"));
if (DEBUG) System.out.println("Done!\n");

if (DEBUG) System.out.print("Reading the input values from INPUT.XML... ");

// --- input values
InputType.ValueType valueElement = inputContent.getValue();
String value  = valueElement.getValue().trim();
String name   = valueElement.getName().trim();

if (DEBUG) 
{
  System.out.println("Done!\n");

  System.out.println("INPUT.XML content:");
  System.out.println("-------------------");

  System.out.println("name:  " + name);
  System.out.println("value: " + value);
}
// ---------------------------------------------------------------------



// ---------------------- connect to DATABASE --------------------------
System.out.print("Connecting to the MeMo database... ");

String query; 
ResultSet rs;
Connection con;
Statement stmt;
boolean more;
Object perhapsNULL;

// --- load the database driver
Class.forName(driver);

// --- connect to the driver
con = DriverManager.getConnection(url, user, password);

if (DEBUG)
{
  System.out.println("Done!\n");

  System.out.println("\nDatabase connection details:");
  System.out.println("----------------------------");

  // --- get the DatabaseMetaData object
  DatabaseMetaData dma = con.getMetaData();

  // --- display some information about the connection
  System.out.println("Connected to: " + dma.getURL());
  System.out.println("Driver:       " + dma.getDriverName());
  System.out.println("Version:      " + dma.getDriverVersion() + "\n");
}

// --- create a Statement object so SQL statements can be submitted to the driver
stmt = con.createStatement();
// ---------------------------------------------------------------------


// -------------------------- test query -------------------------------
if (DEBUG)
{
  System.out.println("Test query:");
  System.out.println("-----------");
  query = "SELECT * from bio_population;";
  System.out.println(query);
  System.out.println("-----------");
  rs = stmt.executeQuery(query);
  more = rs.next();
  while (more)
  {
    System.out.println(rs.getString(1));
    more = rs.next();
  }
  System.out.println();
}
// ---------------------------------------------------------------------



// ---------------------------------------------------------------------
// --- use the ObjectFactory class to create a java content tree from 
//     scratch and marshal it to XML data
// ---------------------------------------------------------------------

// --- create JAXBContext object to provide entry point to JAXB API
JAXBContext jcm = JAXBContext.newInstance("xml.mzdata");

// --- create a Validator
Validator v = jcm.createValidator();

// --- create an ObjectFactory instance
xml.mzdata.ObjectFactory objFactory = new xml.mzdata.ObjectFactory();

// --- create an empty ***MzData*** object
//     <mzData>
xml.mzdata.MzData mzd = objFactory.createMzData();

// --- set the required attributes
mzd.setVersion("1.05");            // --- mzData schema version
mzd.setAccessionNumber(value);     // --- local unique ID for the generated mzData file



// --- create an empty ***Description*** object
//     <mzData> -> <description>
xml.mzdata.MzDataType.DescriptionType description = objFactory.createMzDataTypeDescriptionType();

// ----------------------------------------
// --- create an empty ***Admin*** object
//     <mzData> -> <description> -> <admin>
xml.mzdata.AdminType admin = objFactory.createAdminType();

// --- set the ***sampleName*** element
//     <mzData> -> <description> -> <admin> -> <sampleName>
String sampleID = value;                    // <--- ovo promeniti!!! zavisno od ulaza (if)
admin.setSampleName(sampleID);

// --- create an empty ***SampleDescription*** object
//     <mzData> -> <description> -> <admin> -> <sampleDescription>
xml.mzdata.DescriptionType sampleDescription = objFactory.createDescriptionType();

String sampleDescription_comment = "";  // --- textual description of the metadata, which is 
                                        //     generated based on the values stored in the database

List userParamList0 = sampleDescription.getUserParam();


// --- retrieve sampleDescription given the sample name ... 020508haz01a

// --- biosample: ID & class label

String bioSampleID = "";        // ... 020508haz01a
String class_label = "";        // ... NULL

query = "SELECT BS.id, BS.class_label"          + "\n" +
        "FROM   bio_biosample BS,"              + "\n" +
        "       smp_sample    S"                + "\n" +
        "WHERE  S.id = '" + sampleID + "'"      + "\n" +
        "AND    BS.id = S.biosample_id;";

if (QUERY)
{
  System.out.println("\nRetrieving the BIOLOGICAL SAMPLE information for the given (chemical) sample: " + sampleID);
  System.out.println("-----------");
  System.out.println(query);
  System.out.println("-----------");
}
rs = stmt.executeQuery(query);
more = rs.next();
if (more)
{
  bioSampleID = rs.getString(1);
  if (QUERY) System.out.println("bioSampleID: " + bioSampleID);
  xml.mzdata.UserParamType bioSampleObject = objFactory.createUserParamType();
  bioSampleObject.setName("bioSample");
  bioSampleObject.setValue(bioSampleID);
  userParamList0.add(bioSampleObject);
  sampleDescription_comment += "The given sample has been derived from the following biological sample: " + bioSampleID + ".\n";

  perhapsNULL = rs.getObject(2);
  if(perhapsNULL != null)
  {
    class_label = perhapsNULL.toString();
    if (QUERY) System.out.println("class_label: " + class_label);
    xml.mzdata.UserParamType class_labelObject = objFactory.createUserParamType();
    class_labelObject.setName("class_label");
    class_labelObject.setValue(class_label);
    userParamList0.add(class_labelObject);
    sampleDescription_comment += "This biological sample was classified as: " + class_label + ".\n";
  }
  else {if (QUERY) System.out.println("class_label: NULL");}
}


// --- (biological) source: ID & population

String     sourceID = "";       // ... HAZ152
String populationID = "";       // ... BY4741-2002

query = "SELECT S.id, S.population_id"          + "\n" +
        "FROM   bio_biosample BS,"              + "\n" +
        "       bio_source    S"                + "\n" +
        "WHERE  BS.id = '" + bioSampleID + "'"  + "\n" +
        "AND    BS.source_id = S.id;";

if (QUERY)
{
  System.out.println("\nRetrieving the BIOLOGICAL SOURCE information (ID & population) for the given biosample: " + bioSampleID);
  System.out.println("-----------");
  System.out.println(query);
  System.out.println("-----------");
}
rs = stmt.executeQuery(query);
more = rs.next();
if (more)
{
  sourceID = rs.getString(1);
  xml.mzdata.UserParamType sourceObject = objFactory.createUserParamType();
  sourceObject.setName("source");
  sourceObject.setValue(sourceID);
  userParamList0.add(sourceObject);
  if (QUERY) System.out.println("sourceID: " + sourceID);

  populationID = rs.getString(2);
  xml.mzdata.UserParamType populationObject = objFactory.createUserParamType();
  populationObject.setName("population");
  populationObject.setValue(populationID);
  userParamList0.add(populationObject);
  if (QUERY) System.out.println("populationID: " + populationID);

  sampleDescription_comment += "The biological sample was taken from the following biological source: " + sourceID +
                               ", which is the part of the following population: " + populationID + ".\n";
}


// --- population: organism

String organism = "";           // ... Saccharomyces cerevisiae

query = "SELECT P.organism"                     + "\n" +
        "FROM   bio_population P"               + "\n" +
        "WHERE  P.id = '" + populationID + "';";

if (QUERY)
{
  System.out.println("\nRetrieving the information about the ORGANISM studied given the population: " + populationID);
  System.out.println("-----------");
  System.out.println(query);
  System.out.println("-----------");
}
rs = stmt.executeQuery(query);
more = rs.next();
if (more)
{
  organism = rs.getString(1);
  if (QUERY) System.out.println("organism: " + organism);
  xml.mzdata.UserParamType organismObject = objFactory.createUserParamType();
  organismObject.setName("organism");
  organismObject.setValue(organism);
  userParamList0.add(organismObject);
  sampleDescription_comment += "This population refers to the following organism: " + organism + ".\n";
}


// --- gene knockout: ID

String gene  = "";             // ... YER020W
String genes = "";             // --- in case of multiple knockouts

query = "SELECT G.gene_id"                      + "\n" +
        "FROM   bio_geneknockout G"             + "\n" +
        "WHERE  G.source_id = '" + sourceID + "';";

if (QUERY)
{
  System.out.println("\nRetrieving the GENES knocked out from the given biological source: " + sourceID);
  System.out.println("-----------");
  System.out.println(query);
  System.out.println("-----------");
}
rs = stmt.executeQuery(query);
more = rs.next();
while (more)
{
  gene = rs.getString(1);
  if (QUERY) System.out.println("gene: " + gene);
  xml.mzdata.UserParamType geneObject = objFactory.createUserParamType();
  geneObject.setName("gene");
  geneObject.setValue(gene);
  userParamList0.add(geneObject);
  genes += " " + gene;
  more = rs.next();
}

if (genes.compareTo("") != 0)
sampleDescription_comment += "The following genes were knocked out from the given biological source:" + genes + ".\n";


// --- (chemical) sample: ready date, expiry date, quantity, storage conditions

String ready_date = "";         // ... NULL
String expiry_date = "";        // ... 2002-06-30
String quantity = "";           // ... NULL
String storage_conditions = ""; // ... NULL

query = "SELECT S.ready_date, S.expiry_date, S.quantity, S.storage_conditions"     + "\n" +
        "FROM   smp_sample S"                                                      + "\n" +
        "WHERE  S.id = '" + sampleID + "';";

if (QUERY)
{
  System.out.println("\nRetrieving the data about the given chemical SAMPLE: " + sampleID);
  System.out.println("-----------");
  System.out.println(query);
  System.out.println("-----------");
}
rs = stmt.executeQuery(query);
more = rs.next();
if (more)
{
  perhapsNULL = rs.getObject(1);
  if(perhapsNULL != null)
  {
    ready_date = perhapsNULL.toString();
    if (QUERY) System.out.println("ready_date: " + ready_date);
    xml.mzdata.UserParamType ready_dateObject = objFactory.createUserParamType();
    ready_dateObject.setName("ready_date");
    ready_dateObject.setValue(ready_date);
    userParamList0.add(ready_dateObject);
    sampleDescription_comment += "The sample was prepared on " + ready_date + ". ";
  }
  else {if (QUERY) System.out.println("ready_date: NULL");}

  perhapsNULL = rs.getObject(2);
  if(perhapsNULL != null)
  {
    expiry_date = perhapsNULL.toString();
    if (QUERY) System.out.println("expiry_date: " + expiry_date);
    xml.mzdata.UserParamType expiry_dateObject = objFactory.createUserParamType();
    expiry_dateObject.setName("expiry_date");
    expiry_dateObject.setValue(expiry_date);
    userParamList0.add(expiry_dateObject);
    sampleDescription_comment += "It will expire on " + expiry_date + ".";
  }
  else {if (QUERY) System.out.println("expiry_date: NULL");}

  perhapsNULL = rs.getObject(3);
  if(perhapsNULL != null)
  {
    quantity = perhapsNULL.toString();
    if (QUERY) System.out.println("quantity: " + quantity);
    xml.mzdata.UserParamType quantityObject = objFactory.createUserParamType();
    quantityObject.setName("quantity");
    quantityObject.setValue(quantity);
    userParamList0.add(quantityObject);
    sampleDescription_comment += "Its quantity is " + quantity + "ml. ";
  }
  else {if (QUERY) System.out.println("quantity: NULL");}

  perhapsNULL = rs.getObject(4);
  if(perhapsNULL != null)
  {
    storage_conditions = perhapsNULL.toString();
    if (QUERY) System.out.println("storage_conditions: " + storage_conditions);
    xml.mzdata.UserParamType storage_conditionsObject = objFactory.createUserParamType();
    storage_conditionsObject.setName("storage_conditions");
    storage_conditionsObject.setValue(storage_conditions);
    userParamList0.add(storage_conditionsObject);
    sampleDescription_comment += "It was stored under the following conditions: " + storage_conditions + "\n";
  }
  else {if (QUERY) System.out.println("storage_conditions: NULL");}
}

sampleDescription.setComment(sampleDescription_comment);

if (DEBUG)
{
  System.out.print("\nValidating <mzData> -> <description> -> <admin> -> <sampleDescription>... ");
  OK(v.validate(sampleDescription));
}

admin.setSampleDescription(sampleDescription);


// --- <mzData> -> <description> -> <admin> -> <sourceFile>
xml.mzdata.SourceFileType sourceFile = objFactory.createSourceFileType();

String file_url = "";           // ... NULL
String file_type = "";          // ... NULL

query = "SELECT A.file_url, A.file_type"             + "\n" +
        "FROM   anl_analysis A"                      + "\n" +
        "WHERE  A.sample_id = '" + sampleID + "';";

if (QUERY)
{
  System.out.println("\nRetrieving the URL and the type of the RAW DATA FILE for the analysed sample: " + sampleID);
  System.out.println("-----------");
  System.out.println(query);
  System.out.println("-----------");
}
rs = stmt.executeQuery(query);
more = rs.next();
if (more)
{
  perhapsNULL = rs.getObject(1);
  if(perhapsNULL != null)
  {
    file_url = perhapsNULL.toString();
    if (QUERY) System.out.println("file_url: " + file_url);

    int sep1 = file_url.lastIndexOf('\\') + 1;
    int sep2 = file_url.lastIndexOf('/') + 1;
    int sep = sep1;
    if (sep2 > sep1) sep = sep2;

    String filePath = file_url.substring(0, sep);
    String fileName = file_url.substring(sep);

    sourceFile.setPathToFile(filePath);
    sourceFile.setNameOfFile(fileName);
  }
  else
  {
    if (QUERY) System.out.println("file_url: NULL");

    sourceFile.setNameOfFile("not available");
    sourceFile.setPathToFile("not available");
  }

  perhapsNULL = rs.getObject(2);
  if(perhapsNULL != null)
  {
    file_type = perhapsNULL.toString();
    if (QUERY) System.out.println("file_type: " + file_type);

    sourceFile.setFileType(file_type);
  }
  else {if (QUERY) System.out.println("file_type: NULL");}
}
else
{
  if (QUERY)
  {
    System.out.println("file_url: NULL");
    System.out.println("file_type: NULL");
  }

  sourceFile.setNameOfFile("not available");
  sourceFile.setPathToFile("not available");
}

if (DEBUG)
{
  System.out.print("\nValidating <mzData> -> <description> -> <admin> -> <sourceFile>... ");
  OK(v.validate(sourceFile));
}

admin.setSourceFile(sourceFile);


// --- <mzData> -> <description> -> <admin> -> <contact>
List contactList = admin.getContact();
xml.mzdata.PersonType contact = objFactory.createPersonType();

String personName   = "";         // ... Dr Hazel Davey
String labName      = "";         // ... Institute of Biological Sciences, University of Wales
String address      = "";         // ... 
String phone        = "";         // ... 
String e_mail       = "";         // ... hlr@aber.ac.uk
String contact_info = "";         // --- combines address, phone and/or e-mail

query = "SELECT P.name, L.name, P.address, P.phone, P.e_mail"     + "\n" +
        "FROM   smp_sample S,"                                    + "\n" +
        "       adm_experiment E,"                                + "\n" +
        "       adm_experimentseries ES,"                         + "\n" +
        "       adm_person P,"                                    + "\n" +
        "       adm_lab L"                                        + "\n" +
        "WHERE  S.id = '" + sampleID + "'"                        + "\n" +
        "AND    S.experiment_id = E.id"                           + "\n" +
        "AND    E.series_id = ES.id"                              + "\n" +
        "AND    ES.lab_id = L.id"                                 + "\n" +
        "AND    ES.conductor_id = P.id;";

if (QUERY)
{
  System.out.println("\nRetrieving the CONTACT information for the person who handled the given sample: " + sampleID);
  System.out.println("-----------");
  System.out.println(query);
  System.out.println("-----------");
}
rs = stmt.executeQuery(query);
more = rs.next();
if (more)
{
  personName = rs.getString(1);
  labName    = rs.getString(2);

  contact.setName(personName);
  contact.setInstitution(labName);

  if (QUERY)
  {
    System.out.println("personName: " + personName);
    System.out.println("labName: " + labName);
  }

  perhapsNULL = rs.getObject(3);
  if(perhapsNULL != null)
  {
    address = perhapsNULL.toString();
    if (QUERY) System.out.println("address: " + address);
    contact_info += "address: \n"  + address;
  }
  else {if (QUERY) System.out.println("address: NULL");}

  perhapsNULL = rs.getObject(4);
  if(perhapsNULL != null)
  {
    phone = perhapsNULL.toString();
    if (QUERY) System.out.println("phone: " + phone);
    contact_info += "\nphone: \t"  + phone;
  }
  else {if (QUERY) System.out.println("phone: NULL");}

  perhapsNULL = rs.getObject(5);
  if(perhapsNULL != null)
  {
    e_mail = perhapsNULL.toString();
    if (QUERY) System.out.println("e_mail: " + e_mail);
    contact_info += "\ne_mail: \t" + e_mail;
  }
  else {if (QUERY) System.out.println("e_mail: NULL");}

  contact.setContactInfo(contact_info);

  if (DEBUG)
  {
    System.out.print("\nValidating <mzData> -> <description> -> <admin> -> <contact>... ");
    OK(v.validate(contact));
  }

  contactList.add(contact);
}
else
{
  if (QUERY)
  {
    System.out.println("personName: NULL");
    System.out.println("labName: NULL");
    System.out.println("address: NULL");
    System.out.println("phone: NULL");
    System.out.println("e_mail: NULL");
  }
}

if (DEBUG)
{
  System.out.print("\nValidating <mzData> -> <description> -> <admin>... ");
  OK(v.validate(admin));
}

description.setAdmin(admin);

// --- finished with the ***Admin*** object
//     <mzData> -> <description> -> <admin>
// ----------------------------------------



// ----------------------------------------
// --- create an empty ***Instrument*** object
// --- <mzData> -> <description> -> <instrument>
xml.mzdata.InstrumentDescriptionType instrument = objFactory.createInstrumentDescriptionType();

// --- <mzData> -> <description> -> <instrument> -> <additional>
xml.mzdata.ParamType additional = objFactory.createParamType();
List userParamList4 = additional.getUserParam();

String instrumentID = "";
String manufacturer = "";
String model        = "";
String serialNumber = "";

query = "SELECT I.id, I.manufacturer, I.model, I.serial_number"    + "\n" +
        "FROM   anl_analysis A,"                                   + "\n" +
        "       adm_experiment E,"                                 + "\n" +
        "       adm_instrumentused IU,"                            + "\n" +
        "       adm_instrument I"                                  + "\n" +
        "WHERE  A.sample_id = '" + sampleID + "'"                  + "\n" +
        "AND    A.experiment_id = E.id"                            + "\n" +
        "AND    IU.series_id = E.series_id"                        + "\n" +
        "AND    IU.instrument_id = I.id;";

if (QUERY)
{
  System.out.println("\nRetrieving the information about the ANALYTICAL INSTRUMENT used to analyse the given sample: " + sampleID);
  System.out.println("-----------");
  System.out.println(query);
  System.out.println("-----------");
}
rs = stmt.executeQuery(query);
more = rs.next();
if (more)
{
  instrumentID = rs.getString(1);
  if (QUERY) System.out.println("instrumentID: " + instrumentID);

  perhapsNULL = rs.getObject(2);
  if(perhapsNULL != null)
  {
    manufacturer = perhapsNULL.toString();
    if (QUERY) System.out.println("manufacturer: " + manufacturer);
    xml.mzdata.UserParamType manufacturerObject = objFactory.createUserParamType();
    manufacturerObject.setName("manufacturer");
    manufacturerObject.setValue(manufacturer);
    userParamList4.add(manufacturerObject);
  }
  else {if (QUERY) System.out.println("manufacturer: NULL");}

  perhapsNULL = rs.getObject(3);
  if(perhapsNULL != null)
  {
    model = perhapsNULL.toString();
    if (QUERY) System.out.println("model: " + model);
    xml.mzdata.UserParamType modelObject = objFactory.createUserParamType();
    modelObject.setName("model");
    modelObject.setValue(model);
    userParamList4.add(modelObject);
  }
  else {if (QUERY) System.out.println("model: NULL");}

  perhapsNULL = rs.getObject(4);
  if(perhapsNULL != null)
  {
    serialNumber = perhapsNULL.toString();
    if (QUERY) System.out.println("serialNumber: " + serialNumber);
    xml.mzdata.UserParamType serialNumberObject = objFactory.createUserParamType();
    serialNumberObject.setName("serial_number");
    serialNumberObject.setValue(serialNumber);
    userParamList4.add(serialNumberObject);
  }
  else {if (QUERY) System.out.println("serialNumber: NULL");}
}
else
{
  if (QUERY)
  {
    System.out.println("instrumentID: NULL");
    System.out.println("manufacturer: NULL");
    System.out.println("model: NULL");
    System.out.println("serialNumber: NULL");
  }

  instrumentID = "not available";
}

instrument.setInstrumentName(instrumentID);



// --- retrieving the instrument settings

String parameters    = "";   // --- instrument settings
String xmlSchemaName = "";   // --- XML schema for instrument settings (if in XML!)

query = "SELECT EP.parameters, EP.xml_schema"                      + "\n" +
        "FROM   anl_analysis A,"                                   + "\n" +
        "       adm_experiment E,"                                 + "\n" +
        "       adm_experimentseries ES,"                          + "\n" +
        "       adm_experimentprotocol EP"                         + "\n" +
        "WHERE  A.sample_id = '" + sampleID + "'"                  + "\n" +
        "AND    A.experiment_id = E.id"                            + "\n" +
        "AND    E.series_id = ES.id"                               + "\n" +
        "AND    ES.protocol_id = EP.id;";

if (QUERY)
{
  System.out.println("\nRetrieving the EXPERIMENTAL PROTOCOL used to analyze the given sample: " + sampleID);
  System.out.println("-----------");
  System.out.println(query);
  System.out.println("-----------");
}
rs = stmt.executeQuery(query);
more = rs.next();
if (more)
{
  perhapsNULL = rs.getObject(1);
  if(perhapsNULL != null)
  {
    parameters = perhapsNULL.toString();
    if (QUERY) System.out.println("parameters:\n" + parameters);
  }
  else {if (QUERY) System.out.println("parameters: NULL");}

  perhapsNULL = rs.getObject(2);
  if(perhapsNULL != null)
  {
    xmlSchemaName = perhapsNULL.toString();
    if (QUERY) System.out.println("xmlSchemaName: " + xmlSchemaName);
  }
  else {if (QUERY) System.out.println("xmlSchemaName: NULL");}
}
else
{
  if (QUERY)
  {
    System.out.println("parameters: NULL");
    System.out.println("xmlSchemaName: NULL");
  }
}

if (parameters.compareTo("") != 0)
{
  if (xmlSchemaName.compareTo("") == 0)
  {
    // --- non-XML data
    //     just store everything together in an XML element inside
    //     <mzData> -> <description> -> <instrument> -> <additional>

    xml.mzdata.UserParamType parametersObject = objFactory.createUserParamType();
    parametersObject.setName("instrument_settings");
    parametersObject.setValue(parameters);
    userParamList4.add(parametersObject);
  }
  else
  {
    // --- XML data retrieved for instrument settings
    //     parse it and store the content of individual elements
    //     where appropriate

    // --- PARAMETERS.XML
    FileWriter store = new FileWriter("tmpFiles\\parameters.xml");
    store.write(parameters);
    store.close();
    if (DEBUG) System.out.print("\nPreparing for PARAMETERS.XML unmarshalling... ");

    xmlSchemaName = xmlSchemaName.substring(0, xmlSchemaName.indexOf(".xsd"));
    JAXBContext jcp = JAXBContext.newInstance("xml." + xmlSchemaName);
    Unmarshaller pUnmarshaller = jcp.createUnmarshaller();
    pUnmarshaller.setValidating(true);
    if (DEBUG) System.out.println("Done!");
    if (DEBUG) System.out.println("\nUnmarshalling PARAMETERS.XML... ");

    if (xmlSchemaName.compareTo("masslynx") == 0)
    {
      xml.masslynx.MasslynxRunType parametersXML = 
         (xml.masslynx.MasslynxRunType) pUnmarshaller.unmarshal(new File("tmpFiles\\parameters.xml")); 

      // --- SOURCE parameters
      xml.masslynx.MasslynxRunType.LctTuneType.SourceType sourceXML = (parametersXML.getLctTune()).getSource();

      xml.masslynx.Voltage     extractionCone         = sourceXML.getExtractionCone();
      xml.masslynx.Temperature sourceTemperature      = sourceXML.getSourceTemperature();
      xml.masslynx.Voltage     sampleCone             = sourceXML.getSampleCone();
      xml.masslynx.Voltage     capilary               = sourceXML.getCapilary();
      xml.masslynx.Temperature desolvationTemperature = sourceXML.getDesolvationTemperature();
      xml.masslynx.Voltage     rfLens                 = sourceXML.getRfLens();

      String unit = "";

      // --- <mzData> -> <description> -> <instrument> -> <source>
      xml.mzdata.ParamType source = objFactory.createParamType();
      List userParamList1 = source.getUserParam();

      String extractionConeString = extractionCone.getValue().toString();
      unit = extractionCone.getUnit();
      if (unit.compareTo("") != 0) extractionConeString += " " + unit;

      String sourceTemperatureString = sourceTemperature.getValue().toString();
      unit = sourceTemperature.getUnit();
      if (unit.compareTo("") != 0) sourceTemperatureString += " " + unit;

      String sampleConeString = sampleCone.getValue().toString();
      unit = sampleCone.getUnit();
      if (unit.compareTo("") != 0) sampleConeString += " " + unit;

      String capilaryString = capilary.getValue().toString();
      unit = capilary.getUnit();
      if (unit.compareTo("") != 0) capilaryString += " " + unit;

      String desolvationTemperatureString = desolvationTemperature.getValue().toString();
      unit = desolvationTemperature.getUnit();
      if (unit.compareTo("") != 0) desolvationTemperatureString += " " + unit;

      String rfLensString = rfLens.getValue().toString();
      unit = rfLens.getUnit();
      if (unit.compareTo("") != 0) rfLensString += " " + unit;

      if (DEBUG)
      {
        System.out.println("extractionCone:         " + extractionConeString);
        System.out.println("sourceTemperature:      " + sourceTemperatureString);
        System.out.println("sampleCone:             " + sampleConeString);
        System.out.println("capilary:               " + capilaryString);
        System.out.println("desolvationTemperature: " + desolvationTemperatureString);
        System.out.println("rfLens:                 " + rfLensString);
      }

      xml.mzdata.UserParamType extractionConeObject = objFactory.createUserParamType();
      extractionConeObject.setName("extraction_cone");
      extractionConeObject.setValue(extractionConeString);
      userParamList1.add(extractionConeObject);

      xml.mzdata.UserParamType sourceTemperatureObject = objFactory.createUserParamType();
      sourceTemperatureObject.setName("source_temperature");
      sourceTemperatureObject.setValue(sourceTemperatureString);
      userParamList1.add(sourceTemperatureObject);

      xml.mzdata.UserParamType sampleConeObject = objFactory.createUserParamType();
      sampleConeObject.setName("sample_cone");
      sampleConeObject.setValue(sampleConeString);
      userParamList1.add(sampleConeObject);

      xml.mzdata.UserParamType capilaryObject = objFactory.createUserParamType();
      capilaryObject.setName("capilary");
      capilaryObject.setValue(capilaryString);
      userParamList1.add(capilaryObject);

      xml.mzdata.UserParamType desolvationTemperatureObject = objFactory.createUserParamType();
      desolvationTemperatureObject.setName("desolvation_temperature");
      desolvationTemperatureObject.setValue(desolvationTemperatureString);
      userParamList1.add(desolvationTemperatureObject);

      xml.mzdata.UserParamType rfLensObject = objFactory.createUserParamType();
      rfLensObject.setName("rf_lens");
      rfLensObject.setValue(rfLensString);
      userParamList1.add(rfLensObject);

      instrument.setSource(source);


      // --- ADVANCED parameters
      xml.masslynx.MasslynxRunType.LctTuneType.AdvancedType advancedXML = (parametersXML.getLctTune()).getAdvanced();

      xml.masslynx.Voltage rfDcOffset1     = advancedXML.getRfDcOffset1();
      xml.masslynx.Voltage rfDcOffset2     = advancedXML.getRfDcOffset2();
      xml.masslynx.Voltage aperture        = advancedXML.getAperture();
      xml.masslynx.Voltage acceleration    = advancedXML.getAcceleration();
      xml.masslynx.Voltage focus           = advancedXML.getFocus();
      xml.masslynx.Voltage steering        = advancedXML.getSteering();
      xml.masslynx.Voltage mcpDetector     = advancedXML.getMcpDetector();
      String               pusherCycleTime = advancedXML.getPusherCycleTime();
      String               pusherFrequency = advancedXML.getPusherFrequency().toString();

      String rfDcOffset1String = rfDcOffset1.getValue().toString();
      unit = rfDcOffset1.getUnit();
      if (unit.compareTo("") != 0)  rfDcOffset1String += " " + unit;

      String rfDcOffset2String = rfDcOffset2.getValue().toString();
      unit = rfDcOffset2.getUnit();
      if (unit.compareTo("") != 0) rfDcOffset2String += " " + unit;

      String apertureString = aperture.getValue().toString();
      unit = aperture.getUnit();
      if (unit.compareTo("") != 0) apertureString += " " + unit;

      String accelerationString = acceleration.getValue().toString();
      unit = acceleration.getUnit();
      if (unit.compareTo("") != 0) accelerationString += " " + unit;

      String focusString = focus.getValue().toString();
      unit = focus.getUnit();
      if (unit.compareTo("") != 0) focusString += " " + unit;

      String steeringString = steering.getValue().toString();
      unit = steering.getUnit();
      if (unit.compareTo("") != 0) steeringString += " " + unit;

      String mcpDetectorString = mcpDetector.getValue().toString();
      unit = mcpDetector.getUnit();
      if (unit.compareTo("") != 0) mcpDetectorString += " " + unit;

      if (DEBUG)
      {
        System.out.println("rfDcOffset1:     " + rfDcOffset1String);
        System.out.println("rfDcOffset2:     " + rfDcOffset2String);
        System.out.println("aperture:        " + apertureString);
        System.out.println("acceleration:    " + accelerationString);
        System.out.println("focus:           " + focusString);
        System.out.println("steering:        " + steeringString);
        System.out.println("mcpDetector:     " + mcpDetectorString);
        System.out.println("pusherCycleTime: " + pusherCycleTime);
        System.out.println("pusherFrequency: " + pusherFrequency);
      }

      // --- <mzData> -> <description> -> <instrument> -> <additional>

      xml.mzdata.UserParamType rfDcOffset1Object = objFactory.createUserParamType();
      rfDcOffset1Object.setName("rf_dc_offset_1");
      rfDcOffset1Object.setValue(rfDcOffset1String);
      userParamList1.add(rfDcOffset1Object);

      xml.mzdata.UserParamType rfDcOffset2Object = objFactory.createUserParamType();
      rfDcOffset2Object.setName("rf_dc_offset_2");
      rfDcOffset2Object.setValue(rfDcOffset2String);
      userParamList1.add(rfDcOffset2Object);

      xml.mzdata.UserParamType apertureObject = objFactory.createUserParamType();
      apertureObject.setName("aperture");
      apertureObject.setValue(apertureString);
      userParamList1.add(apertureObject);

      xml.mzdata.UserParamType accelerationObject = objFactory.createUserParamType();
      accelerationObject.setName("acceleration");
      accelerationObject.setValue(accelerationString);
      userParamList1.add(accelerationObject);

      xml.mzdata.UserParamType focusObject = objFactory.createUserParamType();
      focusObject.setName("focus");
      focusObject.setValue(focusString);
      userParamList1.add(focusObject);

      xml.mzdata.UserParamType steeringObject = objFactory.createUserParamType();
      steeringObject.setName("steering");
      steeringObject.setValue(steeringString);
      userParamList1.add(steeringObject);

      xml.mzdata.UserParamType mcpDetectorObject = objFactory.createUserParamType();
      mcpDetectorObject.setName("mcp_detector");
      mcpDetectorObject.setValue(mcpDetectorString);
      userParamList1.add(mcpDetectorObject);

      xml.mzdata.UserParamType pusherCycleObject = objFactory.createUserParamType();
      pusherCycleObject.setName("pusher_cycle_time");
      pusherCycleObject.setValue(pusherCycleTime);
      userParamList1.add(pusherCycleObject);

      xml.mzdata.UserParamType pusherFrequencyObject = objFactory.createUserParamType();
      pusherFrequencyObject.setName("pusher_frequency");
      pusherFrequencyObject.setValue(pusherFrequency);
      userParamList1.add(pusherFrequencyObject);


      // --- ENGINEER parameters
      xml.masslynx.MasslynxRunType.LctTuneType.EngineerType engineerXML = (parametersXML.getLctTune()).getEngineer();

      xml.masslynx.Voltage ionEnergy     = engineerXML.getIonEnergy();
      xml.masslynx.Voltage tubeLens      = engineerXML.getTubeLens();
      xml.masslynx.Voltage grid2         = engineerXML.getGrid2();
      xml.masslynx.Voltage tofFlightTube = engineerXML.getTofFlightTube();
      xml.masslynx.Voltage reflectron    = engineerXML.getReflectron();

      String ionEnergyString     = ionEnergy.getValue().toString();
      unit = ionEnergy.getUnit();
      if (unit.compareTo("") != 0) ionEnergyString += " " + unit;

      String tubeLensString      = tubeLens.getValue().toString();
      unit = tubeLens.getUnit();
      if (unit.compareTo("") != 0) tubeLensString += " " + unit;

      String grid2String         = grid2.getValue().toString();
      unit = grid2.getUnit();
      if (unit.compareTo("") != 0) grid2String += " " + unit;

      String tofFlightTubeString = tofFlightTube.getValue().toString();
      unit = tofFlightTube.getUnit();
      if (unit.compareTo("") != 0) tofFlightTubeString += " " + unit;

      String reflectronString    = reflectron.getValue().toString();
      unit = reflectron.getUnit();
      if (unit.compareTo("") != 0) reflectronString += " " + unit;

      if (DEBUG)
      {
        System.out.println("ionEnergy:     " + ionEnergyString);
        System.out.println("tubeLens:      " + tubeLensString);
        System.out.println("grid2:         " + grid2String);
        System.out.println("tofFlightTube: " + tofFlightTubeString);
        System.out.println("reflectron:    " + reflectronString);
      }

      // --- <mzData> -> <description> -> <instrument> -> <additional>

      xml.mzdata.UserParamType ionEnergyObject = objFactory.createUserParamType();
      ionEnergyObject.setName("ion_energy");
      ionEnergyObject.setValue(ionEnergyString);
      userParamList1.add(ionEnergyObject);

      xml.mzdata.UserParamType tubeLensObject = objFactory.createUserParamType();
      tubeLensObject.setName("tube_lens");
      tubeLensObject.setValue(tubeLensString);
      userParamList1.add(tubeLensObject);

      xml.mzdata.UserParamType grid2Object = objFactory.createUserParamType();
      grid2Object.setName("grid_2");
      grid2Object.setValue(grid2String);
      userParamList1.add(grid2Object);

      xml.mzdata.UserParamType tofFlightTubeObject = objFactory.createUserParamType();
      tofFlightTubeObject.setName("tof_flight_tube");
      tofFlightTubeObject.setValue(tofFlightTubeString);
      userParamList1.add(tofFlightTubeObject);

      xml.mzdata.UserParamType reflectronObject = objFactory.createUserParamType();
      reflectronObject.setName("reflectron");
      reflectronObject.setValue(reflectronString);
      userParamList1.add(reflectronObject);

      instrument.setAdditional(additional);


      // --- <mzData> -> <description> -> <instrument> -> <analyzerList>
      xml.mzdata.InstrumentDescriptionType.AnalyzerListType analyzerList = objFactory.createInstrumentDescriptionTypeAnalyzerListType();
      analyzerList.setCount(1);   // ???
      List analyzerLista = analyzerList.getAnalyzer();
      xml.mzdata.ParamType analyzer = objFactory.createParamType();
      /***
      List userParamList2 = analyzer.getUserParam();

      userParam.setName("name1");
      userParam.setValue("value1");
      userParamList2.add(userParam);

      userParam.setName("name2");
      userParam.setValue("value2");
      userParamList2.add(userParam);
      ***/
      analyzerLista.add(analyzer);

      instrument.setAnalyzerList(analyzerList);


      // --- <mzData> -> <description> -> <instrument> -> <detector>
      xml.mzdata.ParamType detector = objFactory.createParamType();
      List userParamList3 = detector.getUserParam();
      instrument.setDetector(detector);
    }
    else if (xmlSchemaName.compareTo("msmini") == 0)
    {
      xml.msmini.ElectrosprayType parametersXML = 
         (xml.msmini.ElectrosprayType) pUnmarshaller.unmarshal(new File("tmpFiles\\parameters.xml")); 

      // --- SOURCE parameters

      xml.msmini.Voltage     extractionCone         = parametersXML.getExtractionCone();
      xml.msmini.Temperature sourceTemperature      = parametersXML.getSourceTemperature();
      xml.msmini.Voltage     sampleCone             = parametersXML.getSampleCone();
      xml.msmini.Voltage     capilary               = parametersXML.getCapilary();
      xml.msmini.Temperature desolvationTemperature = parametersXML.getDesolvationTemperature();
      xml.msmini.Voltage     rfLens                 = parametersXML.getRfLens();

      // --- <mzData> -> <description> -> <instrument> -> <source>
      xml.mzdata.ParamType source = objFactory.createParamType();
      List userParamList1 = source.getUserParam();

      String unit = "";

      String extractionConeString = "";
      if (extractionCone != null)
      {
        extractionConeString = extractionCone.getValue().toString();
        unit = extractionCone.getUnit();
        if (unit.compareTo("") != 0) extractionConeString += " " + unit;
        if (QUERY) System.out.println("  extractionCone:         " + extractionConeString);

        xml.mzdata.UserParamType extractionConeObject = objFactory.createUserParamType();
        extractionConeObject.setName("extraction_cone");
        extractionConeObject.setValue(extractionConeString);
        userParamList1.add(extractionConeObject);
      }

      String sourceTemperatureString = "";
      if (sourceTemperature != null)
      {
        sourceTemperatureString = sourceTemperature.getValue().toString();
        unit = sourceTemperature.getUnit();
        if (unit.compareTo("") != 0) sourceTemperatureString += " " + unit;
        if (QUERY) System.out.println("  sourceTemperature:      " + sourceTemperatureString);

        xml.mzdata.UserParamType sourceTemperatureObject = objFactory.createUserParamType();
        sourceTemperatureObject.setName("source_temperature");
        sourceTemperatureObject.setValue(sourceTemperatureString);
        userParamList1.add(sourceTemperatureObject);
      }

      if (sampleCone != null)
      {
        String sampleConeString = sampleCone.getValue().toString();
        unit = sampleCone.getUnit();
        if (unit.compareTo("") != 0) sampleConeString += " " + unit;
        if (QUERY) System.out.println("  sampleCone:             " + sampleConeString);

        xml.mzdata.UserParamType sampleConeObject = objFactory.createUserParamType();
        sampleConeObject.setName("sample_cone");
        sampleConeObject.setValue(sampleConeString);
        userParamList1.add(sampleConeObject);
      }

      if (capilary != null)
      {
        String capilaryString = capilary.getValue().toString();
        unit = capilary.getUnit();
        if (unit.compareTo("") != 0) capilaryString += " " + unit;
        if (QUERY) System.out.println("  capilary:               " + capilaryString);

        xml.mzdata.UserParamType capilaryObject = objFactory.createUserParamType();
        capilaryObject.setName("capilary");
        capilaryObject.setValue(capilaryString);
        userParamList1.add(capilaryObject);
      }

      if (desolvationTemperature != null)
      {
        String desolvationTemperatureString = desolvationTemperature.getValue().toString();
        unit = desolvationTemperature.getUnit();
        if (unit.compareTo("") != 0) desolvationTemperatureString += " " + unit;
        if (QUERY) System.out.println("  desolvationTemperature: " + desolvationTemperatureString);

        xml.mzdata.UserParamType desolvationTemperatureObject = objFactory.createUserParamType();
        desolvationTemperatureObject.setName("desolvation_temperature");
        desolvationTemperatureObject.setValue(desolvationTemperatureString);
        userParamList1.add(desolvationTemperatureObject);
      }

      if (rfLens != null)
      {
        String rfLensString = rfLens.getValue().toString();
        unit = rfLens.getUnit();
        if (unit.compareTo("") != 0) rfLensString += " " + unit;
        if (QUERY) System.out.println("  rfLens:                 " + rfLensString);

        xml.mzdata.UserParamType rfLensObject = objFactory.createUserParamType();
        rfLensObject.setName("rf_lens");
        rfLensObject.setValue(rfLensString);
        userParamList1.add(rfLensObject);
      }

      if (DEBUG) System.out.println("Done!");

      if (DEBUG)
      {
        System.out.print("\nValidating <mzData> -> <description> -> <instrument> -> <source>... ");
        OK(v.validate(source));
      }

      instrument.setSource(source);


      // --- <mzData> -> <description> -> <instrument> -> <analyzerList>
      xml.mzdata.InstrumentDescriptionType.AnalyzerListType analyzerList = objFactory.createInstrumentDescriptionTypeAnalyzerListType();
      analyzerList.setCount(1);   // ???
      List analyzerLista = analyzerList.getAnalyzer();
      xml.mzdata.ParamType analyzer = objFactory.createParamType();
      /***
      List userParamList2 = analyzer.getUserParam();

      userParam.setName("name1");
      userParam.setValue("value1");
      userParamList2.add(userParam);

      userParam.setName("name2");
      userParam.setValue("value2");
      userParamList2.add(userParam);
      ***/
      analyzerLista.add(analyzer);

      if (DEBUG)
      {
        System.out.print("\nValidating <mzData> -> <description> -> <instrument> -> <analyzerList>... ");
        OK(v.validate(analyzerList));
      }

      instrument.setAnalyzerList(analyzerList);


      // --- <mzData> -> <description> -> <instrument> -> <detector>
      xml.mzdata.ParamType detector = objFactory.createParamType();
      List userParamList3 = detector.getUserParam();

      if (DEBUG)
      {
        System.out.print("\nValidating <mzData> -> <description> -> <instrument> -> <detector>... ");
        OK(v.validate(detector));
      }

      instrument.setDetector(detector);
    }
    else
    {
      System.out.println("Unsupported xml schema! Program terminated.");
      System.exit(1);
    }
  }
}

if (DEBUG)
{
  System.out.print("\nValidating <mzData> -> <description> -> <instrument>... ");
  OK(v.validate(instrument));
}

description.setInstrument(instrument);

// --- finished with the ***Instrument*** object
// --- <mzData> -> <description> -> <instrument>
// ---------------------------------------------


// ---------------------------------------------
// --- create an empty ***DataProcessing*** object
// --- <mzData> -> <description> -> <dataProcessing>
xml.mzdata.DataProcessingType dataProcessing = objFactory.createDataProcessingType();

xml.mzdata.DataProcessingType.SoftwareType software = objFactory.createDataProcessingTypeSoftwareType();

String software_name = "";
String software_ver  = "";

query = "SELECT A.software_name, A.software_ver"     + "\n" +
        "FROM   anl_analysis A"                      + "\n" +
        "WHERE  A.sample_id = '" + sampleID + "';";

if (QUERY)
{
  System.out.println("\nRetrieving the information about the SOFTWARE used for data processing.");
  System.out.println("-----------");
  System.out.println(query);
  System.out.println("-----------");
}
rs = stmt.executeQuery(query);
more = rs.next();
if (more)
{
  perhapsNULL = rs.getObject(1);
  if(perhapsNULL != null)
  {
    software_name = perhapsNULL.toString();
    if (QUERY) System.out.println("software_name: " + software_name);
    software.setName(software_name);
  }
  else
  {
    if (QUERY) System.out.println("software_name: NULL");
    software.setName("not available");
  }

  perhapsNULL = rs.getObject(2);
  if(perhapsNULL != null)
  {
    software_ver = perhapsNULL.toString();
    if (QUERY) System.out.println("software_ver: " + software_ver);
    software.setVersion(software_ver);
  }
  else
  {
    if (QUERY) System.out.println("software_ver: NULL");
    software.setVersion("not available");
  }
}
else
{
  if (QUERY)
  {
    System.out.println("software_name: NULL");
    System.out.println("software_ver: NULL");
  }

  software.setName("not available");
  software.setVersion("not available");
}

dataProcessing.setSoftware(software);

if (DEBUG)
{
  System.out.print("\nValidating <mzData> -> <description> -> <dataProcessing>... ");
  OK(v.validate(dataProcessing));
}

description.setDataProcessing(dataProcessing);

if (DEBUG)
{
  System.out.print("\nValidating <mzData> -> <description>... ");
  OK(v.validate(description));
}

mzd.setDescription(description);

// --- finished with the ***DataProcessing*** object
// --- <mzData> -> <description> -> <dataProcessing>
// ---------------------------------------------



// ---------------------------------------------
// --- SPECTRUM LIST
// --- create an empty ***SpectrumList*** object
// --- <mzData> -> <spectrumList>

xml.mzdata.MzDataType.SpectrumListType spectrumList = objFactory.createMzDataTypeSpectrumListType();

int count = 0; // --- the number of spectra for the given sample

// --- NB: This ID cannot be used as a spectrum ID in mzData, because 
//         they defined it as integer. Should this change, the retrieved
//         ID should be used.

String spectrumID[] = new String [MAXSPEC];

query = "SELECT MS.id"                                            + "\n" +
        "FROM   smp_sample S,"                                    + "\n" +
        "       anl_analysis A,"                                  + "\n" +
        "       anl_massspectrum MS"                              + "\n" +
        "WHERE  S.id = '" + sampleID + "'"                        + "\n" +
        "AND    S.id = A.sample_id"                               + "\n" +
        "AND    A.id = MS.analysis_id;";

if (QUERY)
{
  System.out.println("\nRetrieving the REFERENCES to all MASS SPECTRA produced for the given sample: " + sampleID);
  System.out.println("-----------");
  System.out.println(query);
  System.out.println("-----------");
  System.out.println("spectrum IDs:");
}
rs = stmt.executeQuery(query);
more = rs.next();
while (more)
{
  spectrumID[count] = rs.getString(1);
  if (QUERY) System.out.println(spectrumID[count]);
  count++;
  more = rs.next();
}

spectrumList.setCount(count);
List spectrumLista = spectrumList.getSpectrum();

while (count > 0)
{
  count--;

  // --- <mzData> -> <spectrumList> -> <spectrum>
  xml.mzdata.MzDataType.SpectrumListType.SpectrumType spectrum = objFactory.createMzDataTypeSpectrumListTypeSpectrumType();
  spectrum.setId(count); // --- Dummy ID! See the above comment about spectrumID.

  // --- <mzData> -> <spectrumList> -> <spectrum> -> <spectrumDesc>
  xml.mzdata.SpectrumDescType spectrumDesc = objFactory.createSpectrumDescType();

  // --- <mzData> -> <spectrumList> -> <spectrum> -> <spectrumDesc> ->  <spectrumSettings>
  xml.mzdata.SpectrumSettingsType spectrumSettings = objFactory.createSpectrumSettingsType();

  // --- <mzData> -> <spectrumList> -> <spectrum> -> <spectrumDesc> ->  <spectrumSettings> -> <spectrumInstrument>
  xml.mzdata.SpectrumSettingsType.SpectrumInstrumentType spectrumInstrument = objFactory.createSpectrumSettingsTypeSpectrumInstrumentType();

  /*** Some of the instrument settings are specified already in 
       <mzData> -> <description> -> <instrument>.
       Not really sure about the difference.
  List userParamList5 = spectrumInstrument.getUserParam();

  userParam.setName("name1");
  userParam.setValue("value1");
  userParamList5.add(userParam);

  userParam.setName("name2");
  userParam.setValue("value2");
  userParamList5.add(userParam);
  ***/

  spectrumInstrument.setMsLevel(1);        // --- for tandem MS ???

  int mz_min = 0, mz_max = 0;
  query = "SELECT MS.mz_min, MS.mz_max"                             + "\n" +
          "FROM   anl_massspectrum MS"                              + "\n" +
          "WHERE  MS.id = '" + spectrumID[count] + "';";

  if (QUERY)
  {
    System.out.println("\nRetrieving the m/z RANGE information for the given mass spectrum: " + spectrumID[count]);
    System.out.println("-----------");
    System.out.println(query);
    System.out.println("-----------");
  }
  rs = stmt.executeQuery(query);
  more = rs.next();
  if (more)
  {
    perhapsNULL = rs.getObject(1);
    if(perhapsNULL != null)
    {
      mz_min = (new Integer(perhapsNULL.toString())).intValue();
      if (QUERY) System.out.println("mz_min: " + mz_min);
      spectrumInstrument.setMzRangeStart(mz_min);
    }
    else {System.out.println("mz_min: NULL");}

    perhapsNULL = rs.getObject(2);
    if(perhapsNULL != null)
    {
      mz_max = (new Integer(perhapsNULL.toString())).intValue();
      if (QUERY) System.out.println("mz_max: " + mz_max);
      spectrumInstrument.setMzRangeStop(mz_max);
    }
    else {System.out.println("mz_max: NULL");}
  }

  if (DEBUG)
  {
    System.out.print("\nValidating <mzData> -> <spectrumList> -> <spectrumDesc> -> <spectrumSettings> -> <spectrumInstrument>... ");
    OK(v.validate(spectrumInstrument));
  }

  spectrumSettings.setSpectrumInstrument(spectrumInstrument);
  
  if (DEBUG)
  {
    System.out.print("\nValidating <mzData> -> <spectrumList> -> <spectrumDesc> -> <spectrumSettings>... ");
    OK(v.validate(spectrumSettings));
  }

  spectrumDesc.setSpectrumSettings(spectrumSettings);

  if (DEBUG)
  {
    System.out.print("\nValidating <mzData> -> <spectrumList> -> <spectrumDesc>... ");
    OK(v.validate(spectrumDesc));
  }

  spectrum.setSpectrumDesc(spectrumDesc);


  // --- MASS SPECTRUM
  // --- X: <mzData> -> <spectrumList> -> <spectrum> -> <mzArrayBinary>
  // --- Y: <mzData> -> <spectrumList> -> <spectrum> -> <IntenArrayBinary>

  xml.mzdata.PeakListBinaryType    mzArrayBinary = objFactory.createPeakListBinaryType();  // --- mz
  xml.mzdata.PeakListBinaryType intenArrayBinary = objFactory.createPeakListBinaryType();  // --- intensity

  xml.mzdata.BinaryDataGroupDataType    mzData = objFactory.createBinaryDataGroupDataType();
  xml.mzdata.BinaryDataGroupDataType intenData = objFactory.createBinaryDataGroupDataType();

  // --- mass spectrum in binary format
  byte        mzBinary[] = new byte [MAXARRAY];
  byte intensityBinary[] = new byte [MAXARRAY];
  int  index             = 0;

  // --- retrieve the count-th spectrum
  query = "SELECT MP.mz_value, MP.absolute_intensity"               + "\n" +
          "FROM   anl_mspeak MP"                                    + "\n" +
          "WHERE  MP.ms_id = '" + spectrumID[count] + "'"           + "\n" +
          "ORDER BY MP.mz_value;";

  if (QUERY)
  {
    System.out.println("\nRetrieving the given MASS SPECTRUM: " + spectrumID[count]);
    System.out.println("-----------");
    System.out.println(query);
    System.out.println("-----------");
    System.out.println("mz\tint");
  }
  rs = stmt.executeQuery(query);
  more = rs.next();
  while (more)
  {
    String mz_value           = rs.getString(1);
    String absolute_intensity = rs.getString(2);

    if (QUERY) System.out.println(mz_value + "\t" + absolute_intensity);

    // --- convert to float and than to int, but bit-wise not by the value
    int mz_value_bits           = Float.floatToRawIntBits( (new Float(mz_value)).floatValue() );
    int absolute_intensity_bits = Float.floatToRawIntBits( (new Float(absolute_intensity)).floatValue() );

    // --- mass peak in binary format
    byte bufferMz[]        = intToByteArray(mz_value_bits);
    byte bufferIntensity[] = intToByteArray(absolute_intensity_bits);

    // --- add current mass peak to its mass spectrum in binary format
    for (int i = 0; i < 4; i++)
    {
             mzBinary[index] = bufferMz[i];
      intensityBinary[index] = bufferIntensity[i];

      index++;
    }

    more = rs.next();
  }

  mzData.setPrecision("32");
  mzData.setEndian("big");
  mzData.setLength(index);

  intenData.setPrecision("32");
  intenData.setEndian("big");
  intenData.setLength(index);

     mzData.setValue(newByteArray(mzBinary, index));
  intenData.setValue(newByteArray(intensityBinary, index));

  if (DEBUG)
  {
    System.out.print("\nValidating <mzData> -> <spectrumList> -> <spectrum> -> <mzArrayBinary> -> <data>... ");
    OK(v.validate(mzData));

    System.out.print("\nValidating <mzData> -> <spectrumList> -> <spectrum> -> <intenArrayBinary> -> <data>... ");
    OK(v.validate(spectrumDesc));
  }

     mzArrayBinary.setData(mzData);
  intenArrayBinary.setData(intenData);

  if (DEBUG)
  {
    System.out.print("\nValidating <mzData> -> <spectrumList> -> <spectrum> -> <mzArrayBinary>... ");
    OK(v.validate(mzArrayBinary));

    System.out.print("\nValidating <mzData> -> <spectrumList> -> <spectrum> -> <intenArrayBinary>... ");
    OK(v.validate(intenArrayBinary));
  }

  spectrum.setMzArrayBinary(mzArrayBinary);
  spectrum.setIntenArrayBinary(intenArrayBinary);

  if (DEBUG)
  {
    System.out.print("\nValidating <mzData> -> <spectrumList> -> <spectrum>... ");
    OK(v.validate(spectrum));
  }

  spectrumLista.add(spectrum);
}

if (DEBUG)
{
  System.out.print("\nValidating <mzData> -> <spectrumList>... ");
  OK(v.validate(spectrumList));
}

mzd.setSpectrumList(spectrumList);

// --- finished with the ***SpectrumList*** object
// --- <mzData> -> <spectrumList>
// ---------------------------------------------


// --- validate the content tree
if (DEBUG)
{
  System.out.print("\nValidating the root element: <mzData>... ");
  OK(v.validateRoot(mzd));
}


// --- marshal the generated mzData (mzd) into to a file (resultsOut)
if (DEBUG) System.out.print("\nMarshalling the XML-formatted results into a file... ");
Marshaller m = jcm.createMarshaller();
m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
FileWriter resultsOut = new FileWriter("output\\mzData_" + sampleID + ".xml");
m.marshal(mzd, resultsOut);
resultsOut.close();
if (DEBUG) System.out.println("Done!");


// ---------------------------- clean up -------------------------------
rs.close();
stmt.close();
con.close();
// ---------------------------------------------------------------------



} // < - - - end of try{...}

// ---------------------------------------------------------------------
catch (JAXBException e)
{
  System.out.println("\n*** JAXBException caught ***\n");
  System.out.println(e.toString());
  e.printStackTrace();
}
// ---------------------------------------------------------------------
catch (ClassNotFoundException cnfe)
{
  System.err.println("Couldn't find driver class:");
  cnfe.printStackTrace();
}
// ---------------------------------------------------------------------
catch (SQLException ex) 
{
  System.out.println ("\n*** SQLException caught ***\n");

  while (ex != null) 
  {
    System.out.println("SQLState: " + ex.getSQLState());
    System.out.println("Message:  " + ex.getMessage());
    System.out.println("Vendor:   " + ex.getErrorCode());
    ex = ex.getNextException();
    System.out.println("");
  }
}
// --------------------------------------------------------------------
catch (java.lang.Exception ex) 
{
  // --- got some other type of exception
  // ---> dump it!

  ex.printStackTrace();
}
// --------------------------------------------------------------------

} // < - - - end of main{...}


// --------------------------------------------------------------------
// --- integer to binary
// --------------------------------------------------------------------
private static byte[] intToByteArray(int number)
{
  byte[] byteArray = new byte[4];

  byteArray[0] = (byte)((number >> 24) & 0xFF);
  byteArray[1] = (byte)((number >> 16) & 0xFF);
  byteArray[2] = (byte)((number >> 8)  & 0xFF);
  byteArray[3] = (byte) (number        & 0xFF);

  return byteArray;
}
// --------------------------------------------------------------------


// -------------------------------------------------------------------
// --- binary array of the given LENGTH
// -------------------------------------------------------------------
private static byte[] newByteArray(byte[] anArray, int dim)
{
  byte nov[] = new byte[dim];

  for (int i = 0; i < dim; i++) nov[i] = anArray[i];

  return nov;
}
// -------------------------------------------------------------------


// -------------------------------------------------------------------
// --- print whether the created mzData document is valid or not
// -------------------------------------------------------------------
private static void OK(boolean valid)
{
  if (valid) System.out.println("OK!");
  else       System.out.println("INVALID!");

  return;
}
// -------------------------------------------------------------------

} // < - - - end of class
